package com.prosperolambda.android.app.hecate;


import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Typeface;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.location.LocationManager;
import android.os.Bundle;
import android.provider.Settings;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.ToggleButton;

public class MainActivity extends Activity implements View.OnClickListener {
	
	private ToggleButton mBoton;
	private TextView mTexto;
	public static final String TAG="ActividadPrincipal";

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		comprobarSistema();
		setContentView(R.layout.activity_main);
		
		//Antes de nada, establece el contexto global de la aplicacin en los objetos Singleton
		UrgenciaDispatcher.getInstance().setContext( this.getApplicationContext() );
		GpsControl.getInstance().setContext( this.getApplicationContext() );
		
		mBoton= (ToggleButton) findViewById(R.id.boton_start);
		mBoton.setOnClickListener(this);
		
		mTexto= (TextView) findViewById(R.id.principal_texto);
		Typeface tf= Typeface.createFromAsset( getAssets(), "fonts/Roboto-Light.ttf");
		mTexto.setTypeface(tf);
		
					
	}

	@Override
	protected void onStart() {
		super.onStart();
		Log.d(TAG, "onStart, servicio "+MonitorService.estaActivo());
		if (MonitorService.estaActivo()) {
			mBoton.setChecked(true);
			mTexto.setText( getString(R.string.principal_texto_encendido) );
		}
		else{
			mTexto.setText( getString(R.string.principal_texto_bienvenida) );
			mBoton.setChecked(false);
		}

	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		//Agrega los elementos del men a la "Action Bar"
		getMenuInflater().inflate(R.menu.menu_opciones, menu);
		return true;
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch(item.getItemId()){
			case R.id.opciones:
				Intent intent= new Intent(this, OpcionesActivity.class);
				startActivity(intent);
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}
	
	/**
	 * Comprueba que tanto GPS como acelermetro lineal estn presentes y
	 * disnponibles. En caso contrario informa del error en forma de AlertDialog
	 * y procede a finalizar la aplicacn.
	 */
	private void comprobarSistema() {
		SensorManager sm = (SensorManager) getSystemService(SENSOR_SERVICE);
		// Comprueba la disponibilidad del acelerador lineal
		if (sm.getDefaultSensor(Sensor.TYPE_LINEAR_ACCELERATION) == null) {
			Log.e(TAG, "Acelerometro lineal NO detectado");
			mostrarError(getString(R.string.error_acelerometro));
		}
		Log.d(TAG, "Rango mximo del Acelerometro Lineal: "+sm.getDefaultSensor(Sensor.TYPE_LINEAR_ACCELERATION).getMaximumRange());
		Log.d(TAG, "Rango mximo del Acelerometro Raw: "+sm.getDefaultSensor(Sensor.TYPE_ACCELEROMETER).getMaximumRange());
		
		LocationManager lm= (LocationManager) getSystemService(LOCATION_SERVICE);
		//Comprueba la disponibilidad del GPS
		if(lm.getProvider(LocationManager.GPS_PROVIDER) ==null){
			Log.e(TAG, "Gps NO detectado");
			mostrarError(getString(R.string.error_gps));
		}
	}
	
	/**
	 * Crear un AlertDialog con el ttulo "Error".
	 * Informa del tipo de error ocurrido (obtenido mediante parmetro)
	 * Un solo botn que al cerrar cierra la aplicacin por completo.
	 * @param error Mensaje de error que mostrar el AlertDialog
	 */
	private void mostrarError(String error){
		AlertDialog.Builder builder= new AlertDialog.Builder(this);
		builder.setMessage(error);
		builder.setTitle(R.string.error);
		builder.setPositiveButton(R.string.error_boton, new DialogInterface.OnClickListener() {
			
			//Al pulsar "Cerrar" finaliza la actividad actual, y por tanto la aplicacin
			@Override
			public void onClick(DialogInterface dialog, int which) {
				finish();				
			}
		});
		
		builder.create();
		builder.show();
		return;
	}

	@Override
	public void onClick(View v) {
		Intent intent=new Intent(this, MonitorService.class);		
		
		if(MonitorService.estaActivo() ){
			stopService(intent);
			mTexto.setText( getString(R.string.principal_texto_bienvenida) );
		}
		else{
			LocationManager lm= (LocationManager) getSystemService(Context.LOCATION_SERVICE);
			/*
			 * Si no est activado el gps se mostrar una alerta por pantalla, con opcin
			 * de ir a la pantalla de configuracin para Activarlo. No inicia el servicio
			 * a menos que el gps est activado.
			 */
			if (lm.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
				startService(intent);
				mTexto.setText( getString(R.string.principal_texto_encendido) );
			}
			else{
				((ToggleButton)v).setChecked(false);
				AlertDialog.Builder builder= new AlertDialog.Builder(this);
				builder.setMessage(R.string.gps_texto);
				builder.setTitle(R.string.gps_titulo);
				builder.setPositiveButton(R.string.gps_activar, new DialogInterface.OnClickListener() {				
					//Al pulsar "Activar" lleva a la pantalla de Opciones de Localizacin
					@Override
					public void onClick(DialogInterface dialog, int which) {
						Intent intent=new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
						startActivity(intent);
					}
				});

				builder.setNegativeButton(R.string.error_boton, new DialogInterface.OnClickListener() {
					//Al pulsar "Cerrar" se cierra la actividad. El usuario no activa el gps y
					//por tanto no puede iniciarse el servicio.
					@Override
					public void onClick(DialogInterface dialog, int which) {
						//finish();						
					}
				});

				builder.create();
				builder.show();
			}
			
		}	
	}
}
