package com.prosperolambda.android.app.hecate;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.location.Location;
import android.telephony.SmsManager;
import android.util.Log;
import android.widget.Toast;

public class SmsUrgencia implements IUrgencia {
	
	public final static String TAG="SmsUrgencia";
	private final static String ENVIADO="Sms Enviado";
	
	public void activarUrgencia(){
		/*
		 * Obtiene el contexto y extrae el nmero de telfono, coordenadas y precision
		 * para poder componer el mensaje a enviar.
		 */
		Context e= UrgenciaDispatcher.getInstance().getContext();
		String numero= UrgenciaDispatcher.getInstance().getNumero();
		double[] posicion= UrgenciaDispatcher.getInstance().getPosicion();
		float precision=UrgenciaDispatcher.getInstance().getPrecision();
		
		/*
		 * Convierte las coordenadas al estndar ISO 6709 +/- DDD.DDDDD
		 */
		String latitud= Location.convert(posicion[0], Location.FORMAT_DEGREES); 
		String longitud= Location.convert(posicion[1], Location.FORMAT_DEGREES);
		
		/*
		 * Compone el mensaje informativo con los datos obtenidos anteriormente
		 */
		String mensaje= e.getString(R.string.urgencia_sms_mensaje, 
				(int)precision, latitud, longitud);
		
		//Reemplaza las "comas" por "puntos". As las coordenadas aparecen como link a google maps
		mensaje=mensaje.replace(',', '.');
		
		/*
		 * Enva el mensaje por Sms en texto plano.
		 */
		PendingIntent pi= PendingIntent.getBroadcast(e, 0, new Intent(ENVIADO), 0);		
		e.registerReceiver(new BroadcastReceiver(){           
			@Override
            public void onReceive(Context arg0, Intent arg1) {
                switch (getResultCode())
                {
                    case Activity.RESULT_OK:
                        Toast.makeText(arg0, R.string.urgencia_sms_correcto, 
                                Toast.LENGTH_SHORT).show();
                        break;
                    case SmsManager.RESULT_ERROR_GENERIC_FAILURE:
                        Toast.makeText(arg0, R.string.urgencia_sms_error_generico, 
                                Toast.LENGTH_SHORT).show();
                        break;
                    case SmsManager.RESULT_ERROR_NO_SERVICE:
                        Toast.makeText(arg0, R.string.urgencia_sms_error_cobertura, 
                                Toast.LENGTH_SHORT).show();
                        break;
                    case SmsManager.RESULT_ERROR_NULL_PDU:
                        Toast.makeText(arg0, R.string.urgencia_sms_error_pdu, 
                                Toast.LENGTH_SHORT).show();
                        break;
                    case SmsManager.RESULT_ERROR_RADIO_OFF:
                        Toast.makeText(arg0, R.string.urgencia_sms_error_avion, 
                                Toast.LENGTH_SHORT).show();
                        break;
                }
            }
        }, new IntentFilter(ENVIADO) );
		
		
		/*
		 * Enva el SMS
		 * Hay que prestar especial atencin a las tildes, NO son aceptadas (ees tampoco)
		 * De lo contario no enva el mensaje y responde con RESULT_ERROR_GENERIC_FAILURE.
		 * 
		 */
		SmsManager sms= SmsManager.getDefault();
		//Al enviar se usa trim() para recortar posibles espacios en blanco antes o despus del nmero
		sms.sendTextMessage(numero.trim(), null, mensaje, pi, null);
		Log.d(TAG, mensaje);
		Log.d(TAG, "Longitud del sms: "+mensaje.length());
		
	}

}
