package com.prosperolambda.android.app.hermes;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import android.util.Log;

/**
 *  Siguiendo una filosofa Productor-Consumidor 
 *  		(http://en.wikipedia.org/wiki/Producer-consumer_problem)
 *  se escribe a fichero trazas (Strgin[]) conforme estas se encolan.
 *  El resto del tiempo el hilo permanece inactivo.
 *  
 * @author Jos Juan Pedreo Manresa
 *
 */
public class FicheroLogger implements Runnable {

	private Thread mHilo;
	private Vector<String[]> mCola;
	private File mFichero;
	private boolean mbActivo;
	
	private final static String TAG= "FicheroLogger";

	public FicheroLogger (File fichero){
		mbActivo=true;
		mFichero=fichero;		
		if(mFichero.exists()) mFichero.delete();
		mCola= new Vector<String[]>();
		mHilo= new Thread(this, "FicheroLogger");
		mHilo.start();
		
	}
	
	public synchronized void agregarTraza(String[] traza){
		mCola.addElement(traza);
		notifyAll();
	}
	
	private synchronized String[] desencolarTraza(){
		while( mCola.isEmpty() ){
			try {
				wait();
			} catch (InterruptedException e) { 
				Log.d(TAG, "Thread interrumpido, comienzael shutdown.", e);
				return null;
			}
		}
		return mCola.remove(0);
	}
	
	/*
	 * Permite desactivar el hilo sin hacer uso de 
	 *  Thread.stop() (deprecado)
	 */
	public void desactivar(){
		Log.d(TAG, "Desactivando el Hilo auxiliar de FicheroLogger");
		mbActivo=false;
		mHilo.interrupt();
	}
	
	public void run() {
		
		BufferedWriter bw=null;
		
		while(mbActivo){
			
			String[] aux=desencolarTraza(); //Desencola el primer elemento de la lista.
			try{
				/*
				 * El fichero de escritura se abre en modo append para no sobreescribirlo en cada iteracin.
				 */
				FileWriter fw=new FileWriter(mFichero, true);
				bw= new BufferedWriter(fw);
				
				for(String s: aux){
					if(s!=null) bw.append(s);
				}
				
				bw.close();
			}catch(IOException e) { Log.e(TAG, "Error de escritura en el fichero.", e); }
			 catch(NullPointerException e){ Log.e(TAG, "NullPointerException!"); }
			
		}
	}
		
}
