package com.prosperolambda.android.app.hermes;

import java.io.File;
import java.util.Calendar;

import org.apache.http.protocol.HTTP;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.PowerManager;
import android.os.RemoteException;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends Activity implements ServiceConnection, Callback {
	
	private boolean mbActivo;
	private TextView mPanelTexto;
	private Button mBotonStart;
	private Button mBotonEnviar;
	private Messenger mServicio;
	private Messenger mActivity;
	private PowerManager mPoweManager;
	private PowerManager.WakeLock mWakelock;
	
	public static final String ESTADO_BOTON="botonEnviar";
	public static final String ESTADO_PANEL="panelTexto";
	public static final String TAG="MainActivity";
	public static final String EMAIL="jjpedreno@gmail.com"; //TODO Cambiar por razones de privacidad
	public static final int ENVIAR_EMAIL=1;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
                
        comprobarSistema();
        
        mPanelTexto= (TextView) findViewById(R.id.textPanel);
        mBotonStart= (Button) findViewById(R.id.botonStart);
        mBotonEnviar= (Button) findViewById(R.id.botonEnviar);
        
        mActivity =new Messenger(new Handler(this));
        
        /*
         * Restaura el estado anterior en caso de haberse guardado
         */
        if(savedInstanceState!=null){
        	mbActivo=savedInstanceState.getBoolean(ESTADO_BOTON);
        	mPanelTexto.setText( savedInstanceState.getString(ESTADO_PANEL) );
        	
        	if(mbActivo) {
        		mBotonStart.setText( getString(R.string.botonStop) );
        		conectarServicio();
        	}
        }
        else mbActivo=false;
        
        mBotonStart.setOnClickListener(new View.OnClickListener() {
			
			public void onClick(View v) {
				if(mbActivo){
					Log.d(TAG, "STOP pulsado.");
					mbActivo=false;
					//Se cambia el texto del boton a "Start"
					mBotonStart.setText(getString(R.string.botonStart));
					desconectarServicio();
				}
				else{
					mbActivo=true;
					Log.d(TAG, "START pulsado");
					//Se cambia el texto del boton a "Stop"
					mBotonStart.setText(getString(R.string.botonStop));
					//Se cambia el texto del panel, informando de la actividad
					mPanelTexto.setText(getString(R.string.mensajeMonitorizando));
					conectarServicio();
				}
				
			}
		});
        mBotonEnviar.setOnClickListener(new View.OnClickListener() {
			
			public void onClick(View v) {
				if(!mbActivo){
					//Se crea "Intent" con accin de enviar un email.
					Intent intent= new Intent(Intent.ACTION_SEND);
					//Se establece formato de envo en texto plano.
					intent.setType(HTTP.PLAIN_TEXT_TYPE);
					//Se aade destinatario.
					intent.putExtra(Intent.EXTRA_EMAIL, new String[] { EMAIL } );
					//Se aade subject, con da mes y ao.
					intent.putExtra(Intent.EXTRA_SUBJECT, String.format("Registro del acelermetro, " +
							"%1$td de %1$tB - %1tY", Calendar.getInstance() ) );
					
					/*
					 * Se adjunta fichero con el registro de actividad.
					 * Se obtiene el path del almacenamiento externo mediante getExternFilesDir(). 
					 * En caso de no existir el archivo no se produce error.
					 */
					File fichero= new File( getExternalFilesDir(null), getString(R.string.ficheroLog) );
					Log.d(TAG,"Adjuntando el fichero: "+ fichero.getAbsolutePath() );
					intent.putExtra(Intent.EXTRA_STREAM, Uri.fromFile(fichero) );
					intent.putExtra(Intent.EXTRA_TEXT, mPanelTexto.getText() );
					
					Log.d(TAG, "registo_acelerometro.txt -> EMAIL");
					startActivity(intent);
				}
				
				
			}
		});    
    }
    
    @Override
    protected void onDestroy() {
    	if( isFinishing() && mbActivo ){
    		Log.d(TAG, "Destruyendo Actividad Principal por BackButton");
    		desconectarServicio();
    	}
    	Log.d(TAG, "Destruyendo Actividad Principal");
    	super.onDestroy();
    }
    
    @Override
    protected void onSaveInstanceState(Bundle outState) {
    	super.onSaveInstanceState(outState);
    	//Salva el estado de la actividad
    	outState.putBoolean(ESTADO_BOTON, mbActivo);
    	outState.putString(ESTADO_PANEL, mPanelTexto.getText().toString() ); 
    	Log.d(TAG, "Salvando estado de variables debido a onDestroy");
    	
    }
    
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data){
    }
  
    private void conectarServicio(){
    	if(mPoweManager==null){
    		mPoweManager= (PowerManager) getSystemService(Context.POWER_SERVICE);
    		mWakelock= mPoweManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "WakelockServicio");
    	}
    	
    	mWakelock.acquire();
    	
    	Intent intent=new Intent(this, ServicioRegistroSensores.class);
    	if(mbActivo){
    		Log.d(TAG, "Arranca el servicio");
    		startService(intent);
    		//Indica brevemente por pantalla que el servicio ha sido linkado satisfactoriamente.
    		Toast.makeText(this, getString(R.string.toastServicioArrancado), Toast.LENGTH_SHORT).show();
    	}   	
    	bindService(intent, this, 0);    	
    	Log.d(TAG, "Linka al servicio");
    }
    
    private void desconectarServicio(){
    	mWakelock.release();
    	Intent intent = new Intent(this, ServicioRegistroSensores.class);
    	unbindService(this);
    	stopService(intent);
    	//Informa que se ha detenido el servicio satisfactoriamente
    	Toast.makeText(this, getString(R.string.toastServicioDetenido), Toast.LENGTH_SHORT).show();
    	
    }

	public void onServiceConnected(ComponentName name, IBinder service) {
		//Se crea Messenger con destino el servicio.
		mServicio=new Messenger(service);
		Message mensaje= new Message();
		//Se crea el mensaje con la intencin de conectar.
		mensaje.what=ServicioRegistroSensores.MENSAJE_CONECTAR;
		mensaje.replyTo=mActivity;
		try {
			mServicio.send(mensaje);
		} catch (RemoteException e) {
			Log.e(TAG, "Excepcin durante el envo de CONECTAR");
		}
	}

	public void onServiceDisconnected(ComponentName name) {
	}

	public boolean handleMessage(Message msg) {
		switch (msg.what) {
		case ServicioRegistroSensores.MENSAJE_VALORES:
			float[] valores= (float[]) msg.obj;
			mPanelTexto.setText("");
			mPanelTexto.append( getString(R.string.mensajeLog) + 
					String.format(" %1$tA %1$td de %1$tB, %1$tY:\n\n", Calendar.getInstance() ) );
			mPanelTexto.append( getString(R.string.mensajeEjeX) + 
					String.format("%.2f ( %.1fG)\n", valores[0], valores[0]/SensorManager.GRAVITY_EARTH));
			
			mPanelTexto.append( getString(R.string.mensajeEjeY) + 
					String.format("%.2f ( %.1fG)\n", valores[1], valores[1]/SensorManager.GRAVITY_EARTH));
			
			mPanelTexto.append( getString(R.string.mensajeEjeZ) + 
					String.format("%.2f ( %.1fG)\n", valores[2], valores[2]/SensorManager.GRAVITY_EARTH));
			Log.d(TAG, "Valores de aceleraciones mximas impresos por pantalla");
			break;

		default:
			break;
		}
		return false;
	}
	
	private void comprobarSistema() {
		
		Log.d(TAG, "Comprobando SENSOR y ALMACENAMIENTO");
		SensorManager sensor= (SensorManager) getSystemService(SENSOR_SERVICE);		
        if(sensor.getDefaultSensor(Sensor.TYPE_LINEAR_ACCELERATION) == null ){
        	Log.e(TAG, "Acelerometro no detectado");
        	mostrarError( getString(R.string.errorAcelerometro) );
        	return;
        }
        /*
         * Procede a comprobar que el almacenamiento externo est disponible,
         * montado, y con acceso de escritura.
         */
        boolean existeAlmacenamiento=false;
        boolean escrituraAlmacenamiento=false;
        String estado= Environment.getExternalStorageState();
        
        if( Environment.MEDIA_MOUNTED.equals(estado) ){
        	//Medio montado y con permiso de escritura
        	existeAlmacenamiento=escrituraAlmacenamiento= true;
        }
        else if( Environment.MEDIA_MOUNTED_READ_ONLY.equals(estado) ){
        	//Medio montado pero en modo SOLO LECTURA
        	existeAlmacenamiento=true;
        	escrituraAlmacenamiento=false;
        }
        
        if( !(existeAlmacenamiento && escrituraAlmacenamiento) ){
        	Log.e(TAG, "Error al acceder al almacenamiento externo");
        	mostrarError( getString(R.string.errorAlmacenamiento) );
        }       
        return;
	}
	
	public void mostrarError(String mensaje){
		Intent intent = new Intent(this, ErrorActivity.class);
		intent.putExtra(ErrorActivity.EXTRA_MENSAJE, mensaje);
		startActivity(intent);
		//Cierra esta actividad despus de arrancar el error.
		finish();
	}
}
